@@ -19,6 +19,7 @@
  * modified by Denis Pruchkovsky
  */
 
+#include "config.h"
 #ifdef HAVE_CTYPE_H
 #  include <ctype.h>
 #endif
@@ -94,7 +95,7 @@
           char *s2 = strrplc (s1, "[rcount2]", "[%d]");                       \
           if (s2)                                                             \
             {                                                                 \
-              GCC46_DIAG_IGNORE (-Wformat-nonliteral)                       \
+              GCC46_DIAG_IGNORE (-Wformat-nonliteral)                         \
               LOG_TRACE (strcat (s2, ": " FORMAT_##type " " #type "\n"),      \
                          rcount1, rcount2, _obj->nam)                         \
               GCC46_DIAG_RESTORE                                              \
@@ -103,7 +104,7 @@
             }                                                                 \
           else                                                                \
             {                                                                 \
-              GCC46_DIAG_IGNORE (-Wformat-nonliteral)                       \
+              GCC46_DIAG_IGNORE (-Wformat-nonliteral)                         \
               LOG_TRACE (strcat (s1, ": " FORMAT_##type " " #type "\n"),      \
                          rcount1, _obj->nam)                                  \
               GCC46_DIAG_RESTORE                                              \
@@ -133,6 +134,7 @@
   }
 #define LOG_TRACE_TF(var, len) LOG_TF (TRACE, var, len)
 #define LOG_INSANE_TF(var, len) LOG_TF (INSANE, var, len)
+
 #define FIELD_2PT_TRACE(nam, type, dxf)                                       \
   if (DWG_LOGLEVEL >= DWG_LOGLEVEL_TRACE)                                     \
     {                                                                         \
@@ -142,7 +144,7 @@
           char *s2 = strrplc (s1, "[rcount2]", "[%d]");                       \
           if (s2)                                                             \
             {                                                                 \
-              GCC46_DIAG_IGNORE (-Wformat-nonliteral)                       \
+              GCC46_DIAG_IGNORE (-Wformat-nonliteral)                         \
               LOG_TRACE (strcat (s2, ": (" FORMAT_BD ", " FORMAT_BD           \
                                      ") [" #type " %d]\n"),                   \
                          rcount1, rcount2, _obj->nam.x, _obj->nam.y, dxf)     \
@@ -152,7 +154,7 @@
             }                                                                 \
           else                                                                \
             {                                                                 \
-              GCC46_DIAG_IGNORE (-Wformat-nonliteral)                       \
+              GCC46_DIAG_IGNORE (-Wformat-nonliteral)                         \
               LOG_TRACE (strcat (s1, ": (" FORMAT_BD ", " FORMAT_BD           \
                                      ") [" #type " %d]\n"),                   \
                          rcount1, _obj->nam.x, _obj->nam.y, dxf)              \
@@ -374,7 +376,8 @@
   {                                                                           \
     SINCE (R_13) { _obj->nam = NULL; VECTOR_CHKCOUNT (nam, TF, len, dat) }    \
     _obj->nam = bit_read_TF (dat, (int)len);                                  \
-    LOG_TRACE (#nam ": \"%s\" [TF %lu " #dxf "]\n", _obj->nam, (unsigned long)len); \
+    LOG_TRACE (#nam ": \"%s\" [TF %lu " #dxf "]\n", _obj->nam,                \
+               (unsigned long)len);                                           \
     LOG_INSANE_TF (FIELD_VALUE (nam), (int)len);                              \
   }
 #define FIELD_TFF(nam, len, dxf)                                              \
@@ -419,7 +422,12 @@
   }
 
 #define FIELD_BE(nam, dxf)                                                    \
-  bit_read_BE (dat, &_obj->nam.x, &_obj->nam.y, &_obj->nam.z)
+  {                                                                           \
+    bit_read_BE (dat, &_obj->nam.x, &_obj->nam.y, &_obj->nam.z);              \
+    if (_obj->nam.z != 1.0)                                                   \
+        LOG_TRACE (#nam ": (%f, %f, %f) [BE %d]\n", _obj->nam.x, _obj->nam.y, \
+                   _obj->nam.z, dxf);                                         \
+  }
 #define FIELD_DD(nam, _default, dxf)                                          \
   {                                                                           \
     FIELD_VALUE (nam) = bit_read_DD (dat, _default);                          \
@@ -599,11 +607,12 @@
           LOG_TRACE (#o "." #color ".rgb: 0x%06x [ENC.BL %d]\n",                     \
                      (unsigned)_obj->o.color.rgb, dxf2);                      \
         if (_obj->o.color.flag & 0x40 && _obj->o.color.handle)                \
-          LOG_TRACE (#o "." #color ".handle: %lX [ENC.H %d]\n",                      \
+          LOG_TRACE (#o "." #color ".handle: %lX [ENC.H %d]\n",               \
                      _obj->o.color.handle->handleref.value, dxf2 + 10);       \
       }                                                                       \
   }
 
+
 #undef DEBUG_POS
 #undef DEBUG_HERE
 #undef DEBUG_POS_OBJ
@@ -913,6 +922,8 @@
       {                                                                       \
         VALUE_HANDLE (obj->tio.object->xdicobjhandle, xdicobjhandle, code,    \
                       360);                                                   \
+        if (!obj->tio.object->xdicobjhandle)                                  \
+          obj->tio.object->xdic_missing_flag = 1;                             \
       }                                                                       \
   }                                                                           \
   else                                                                        \
@@ -986,8 +997,7 @@
   LOG_INSANE ("REPEAT_CHKCOUNT %s." #name " x %ld: %lld > %lld?\n",           \
               SAFEDXFNAME, (long)times, (long long)((times) * sizeof (type)), \
               AVAIL_BITS (dat));                                              \
-  if (dat->version >= R_2004                                                  \
-      && (long long)((times) * sizeof (type)) > AVAIL_BITS (dat))             \
+  if ((long long)((times) * sizeof (type)) > AVAIL_BITS (dat))                \
     {                                                                         \
       LOG_ERROR ("Invalid %s." #name " x %ld\n", SAFEDXFNAME, (long)times);   \
       if (_obj->name) { free (_obj->name); _obj->name = NULL; }               \
@@ -1004,8 +1014,7 @@
   LOG_INSANE ("REPEAT_CHKCOUNT_LVAL %s." #name " x %ld: %lld > %lld?\n",      \
               SAFEDXFNAME, (long)times, (long long)((times) * sizeof (type)), \
               AVAIL_BITS (dat));                                              \
-  if (dat->version >= R_2004                                                  \
-      && (long long)((times) * sizeof (type)) > AVAIL_BITS (dat))             \
+  if ((long long)((times) * sizeof (type)) > AVAIL_BITS (dat))                \
     {                                                                         \
       LOG_ERROR ("Invalid %s." #name " x %ld\n", SAFEDXFNAME, (long)times);   \
       times = 0;                                                              \
@@ -1129,19 +1138,22 @@
     if (error)                                                                \
       return error;                                                           \
     SINCE (R_2007)                                                            \
-    {                                                                         \
-      Bit_Chain obj_dat, str_dat, hdl_dat;                                    \
-      obj_dat = *dat;                                                         \
-      hdl_dat = *dat;                                                         \
-      str_dat = *dat;                                                         \
-      error                                                                   \
-          = dwg_decode_##token##_private (&obj_dat, &hdl_dat, &str_dat, obj); \
-    }                                                                         \
-    else { error = dwg_decode_##token##_private (dat, dat, dat, obj); }       \
+      {                                                                       \
+        Bit_Chain obj_dat, str_dat, hdl_dat;                                  \
+        obj_dat = *dat;                                                       \
+        hdl_dat = *dat;                                                       \
+        str_dat = *dat;                                                       \
+        error = dwg_decode_##token##_private (&obj_dat, &hdl_dat,             \
+                                              &str_dat, obj);                 \
+      }                                                                       \
+    else                                                                      \
+      {                                                                       \
+        error = dwg_decode_##token##_private (dat, dat, dat, obj);            \
+      }                                                                       \
     return error;                                                             \
   }                                                                           \
                                                                               \
-  GCC30_DIAG_IGNORE (-Wformat-nonliteral)                                   \
+  GCC30_DIAG_IGNORE (-Wformat-nonliteral)                                     \
   static int dwg_decode_##token##_private (                                   \
       Bit_Chain *dat, Bit_Chain *hdl_dat, Bit_Chain *str_dat,                 \
       Dwg_Object *restrict obj)                                               \
@@ -1228,8 +1240,8 @@
       obj_dat = *dat;                                                         \
       hdl_dat = *dat;                                                         \
       str_dat = *dat;                                                         \
-      error                                                                   \
-          = dwg_decode_##token##_private (&obj_dat, &hdl_dat, &str_dat, obj); \
+      error = dwg_decode_##token##_private (&obj_dat, &hdl_dat, &str_dat,     \
+                                            obj);                             \
     }                                                                         \
     else { error = dwg_decode_##token##_private (dat, dat, dat, obj); }       \
     return error;                                                             \
