@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2010-2019 Free Software Foundation, Inc.                   */
+/*  Copyright (C) 2010-2020 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -28,6 +28,8 @@
 #include "dec_macros.h"
 #include "decode.h"
 
+#define MIN(X, Y) ((X) < (Y) ? (X) : (Y))
+
 /* The logging level for the read (decode) path.  */
 static unsigned int loglevel;
 /* the current version per spec block */
@@ -170,13 +172,14 @@ static void copy_bytes (BITCODE_RC *dst, uint32_t length, uint32_t offset);
 static uint32_t read_literal_length (BITCODE_RC *restrict *restrict src,
                                      unsigned char opcode);
 static void copy_compressed_bytes (BITCODE_RC *restrict dst,
-                                   BITCODE_RC *restrict src, int length);
+                                   BITCODE_RC *restrict src,
+                                   int length);
 static DWGCHAR *bfr_read_string (BITCODE_RC *restrict *restrict src,
                                  int64_t size) ATTRIBUTE_MALLOC;
 static BITCODE_RC *decode_rs (const BITCODE_RC *src, int block_count,
-                              int data_size) ATTRIBUTE_MALLOC;
-static int decompress_r2007 (BITCODE_RC *restrict dst, int dst_size,
-                             BITCODE_RC *restrict src, int src_size);
+                              int data_size, const unsigned src_size) ATTRIBUTE_MALLOC;
+static int decompress_r2007 (BITCODE_RC *restrict dst, const unsigned dst_size,
+                             BITCODE_RC *restrict src, const unsigned src_size);
 
 #define copy_1(offset) *dst++ = *(src + offset);
 
@@ -473,8 +476,8 @@ read_instructions (BITCODE_RC *restrict *src, unsigned char *restrict opcode,
    TODO: replace by decompress_R2004_section(dat, decomp, comp_data_size)
 */
 static int
-decompress_r2007 (BITCODE_RC *restrict dst, int dst_size,
-                  BITCODE_RC *restrict src, int src_size)
+decompress_r2007 (BITCODE_RC *restrict dst, const unsigned dst_size,
+                  BITCODE_RC *restrict src, const unsigned src_size)
 {
   uint32_t length = 0;
   uint32_t offset = 0;
@@ -484,19 +487,18 @@ decompress_r2007 (BITCODE_RC *restrict dst, int dst_size,
   BITCODE_RC *src_end = src + src_size;
   unsigned char opcode;
 
-  if (!src)
+  LOG_INSANE ("decompress_r2007 (%p, %d, %p, %d)\n", dst, dst_size, src, src_size);
+  if (!dst || !src || !dst_size || src_size < 2)
     {
-      LOG_ERROR ("Empty src argument to %s\n", __FUNCTION__);
+      LOG_ERROR ("Invalid argument to %s\n", __FUNCTION__);
       return DWG_ERR_INTERNALERROR;
     }
-  opcode = *src++;
-  LOG_INSANE ("decompress_r2007(%p %d %p %d)\n", dst, dst_size, src, src_size);
 
+  opcode = *src++;
   if ((opcode & 0xf0) == 0x20)
     {
       src += 2;
       length = *src++ & 0x07;
-
       if (length == 0)
         {
           LOG_ERROR ("Decompression error: zero length")
@@ -509,13 +511,13 @@ decompress_r2007 (BITCODE_RC *restrict dst, int dst_size,
       if (length == 0)
         length = read_literal_length (&src, opcode);
 
-      if ((dst + length) > dst_end)
+      if ((dst + length) > dst_end || (src + length) > src_end)
         {
           LOG_ERROR ("Decompression error: length overflow");
           return DWG_ERR_INTERNALERROR;
         }
 
-      // LOG_INSANE("copy_compressed_bytes(%p %p %u)\n", dst, src, length);
+      LOG_INSANE("copy_compressed_bytes (%p, %p, %u)\n", dst, src, length);
       copy_compressed_bytes (dst, src, length);
 
       dst += length;
@@ -542,7 +544,7 @@ decompress_r2007 (BITCODE_RC *restrict dst, int dst_size,
               LOG_ERROR ("Decompression error: offset underflow");
               return DWG_ERR_INTERNALERROR;
             }
-          LOG_INSANE ("copy_bytes(%p %u %u)\n", dst, length, offset);
+          LOG_INSANE ("copy_bytes (%p, %u, %u)\n", dst, length, offset);
           copy_bytes (dst, length, offset);
 
           dst += length;
@@ -569,14 +571,22 @@ decompress_r2007 (BITCODE_RC *restrict dst, int dst_size,
 
 // reed-solomon (255, 239) encoding with factor 3
 // TODO: for now disabled, until we get proper data
+ATTRIBUTE_MALLOC
 static BITCODE_RC *
-decode_rs (const BITCODE_RC *src, int block_count, int data_size)
+decode_rs (const BITCODE_RC *src, int block_count, int data_size,
+           const unsigned src_size)
 {
   int i, j;
   const BITCODE_RC *src_base = src;
   BITCODE_RC *dst_base, *dst;
   // TODO: round up data_size from 239 to 255
 
+  if ((unsigned long)block_count * data_size > src_size)
+    {
+      LOG_ERROR ("decode_rs src overflow: %ld > %u",
+                 (long)block_count * data_size, src_size)
+      return NULL;
+    }
   dst_base = dst = (BITCODE_RC *)calloc (block_count, data_size);
   if (!dst)
     {
@@ -609,6 +619,7 @@ read_system_page (Bit_Chain *dat, int64_t size_comp, int64_t size_uncomp,
   int64_t pesize;      // Pre RS encoded size
   int64_t block_count; // Number of RS encoded blocks
   int64_t page_size;
+  long pedata_size;
 
   BITCODE_RC *rsdata; // RS encoded data
   BITCODE_RC *pedata; // Pre RS encoded data
@@ -641,7 +652,8 @@ read_system_page (Bit_Chain *dat, int64_t size_comp, int64_t size_uncomp,
 
   rsdata = &data[size_uncomp];
   bit_read_fixed (dat, rsdata, page_size);
-  pedata = decode_rs (rsdata, block_count, 239);
+  pedata_size = block_count * 239;
+  pedata = decode_rs (rsdata, block_count, 239, page_size);
   if (!pedata)
     {
       free (data);
@@ -649,7 +661,7 @@ read_system_page (Bit_Chain *dat, int64_t size_comp, int64_t size_uncomp,
     }
 
   if (size_comp < size_uncomp)
-    error = decompress_r2007 (data, size_uncomp, pedata, size_comp);
+    error = decompress_r2007 (data, size_uncomp, pedata, MIN (pedata_size, size_comp));
   else
     memcpy (data, pedata, size_uncomp);
   free (pedata);
@@ -674,10 +686,12 @@ read_data_page (Bit_Chain *restrict dat, BITCODE_RC *restrict decomp,
 
   BITCODE_RC *rsdata; // RS encoded data
   BITCODE_RC *pedata; // Pre RS encoded data
+  long pedata_size;
 
   // Round to a multiple of 8
   pesize = ((size_comp + 7) & ~7);
   block_count = (pesize + 0xFB - 1) / 0xFB;
+  pedata_size = block_count * 0xFB;
 
   rsdata = (BITCODE_RC *)calloc (1, page_size);
   if (rsdata == NULL)
@@ -686,12 +700,13 @@ read_data_page (Bit_Chain *restrict dat, BITCODE_RC *restrict decomp,
       return DWG_ERR_OUTOFMEM;
     }
   bit_read_fixed (dat, rsdata, page_size);
-  pedata = decode_rs (rsdata, block_count, 0xFB);
+  pedata = decode_rs (rsdata, block_count, 0xFB, page_size);
   if (!pedata)
     return DWG_ERR_OUTOFMEM;
 
   if (size_comp < size_uncomp)
-    error = decompress_r2007 (decomp, size_uncomp, pedata, size_comp);
+    error = decompress_r2007 (decomp, size_uncomp, pedata,
+                              MIN (pedata_size, size_comp));
   else
     memcpy (decomp, pedata, size_uncomp);
 
@@ -720,6 +735,11 @@ read_data_section (Bit_Chain *sec_dat, Bit_Chain *dat,
     }
 
   max_decomp_size = section->data_size;
+  if (max_decomp_size > 0x2f000000) // 790Mb
+    {
+      LOG_ERROR ("Invalid max decompression size %" PRIu64, max_decomp_size);
+      return DWG_ERR_INVALIDDWG;
+    }
   decomp = calloc (max_decomp_size, 1);
   if (decomp == NULL)
     {
@@ -913,6 +933,11 @@ read_sections_map (Bit_Chain *dat, int64_t size_comp, int64_t size_uncomp,
 
       if (section->num_pages <= 0)
         continue;
+      if (section->num_pages > 0xf0000)
+        {
+          LOG_ERROR ("Invalid num_pages %lu, skip", (unsigned long)section->num_pages);
+          continue;
+        }
 
       section->pages = (r2007_section_page **)calloc (
           (size_t)section->num_pages, sizeof (r2007_section_page *));
@@ -940,6 +965,12 @@ read_sections_map (Bit_Chain *dat, int64_t size_comp, int64_t size_uncomp,
               return NULL;
             }
 
+          if (ptr + 56 >= ptr_end)
+            {
+              LOG_ERROR ("Section[%d]->pages[%d] overflow", j, i);
+              section->num_pages = i; // skip this last section
+              break;
+            }
           bfr_read (section->pages[i], &ptr, 56);
 
           LOG_TRACE (" Page[%d]: ", i)
@@ -952,6 +983,14 @@ read_sections_map (Bit_Chain *dat, int64_t size_comp, int64_t size_uncomp,
           LOG_HANDLE (" checksum: %016" PRIx64, section->pages[i]->checksum);
           LOG_HANDLE (" crc64: %016" PRIx64 "\n", section->pages[i]->crc);
           // debugging sanity
+          if (section->pages[i]->size >= DBG_MAX_SIZE
+              || section->pages[i]->uncomp_size >= DBG_MAX_SIZE
+              || section->pages[i]->comp_size >= DBG_MAX_SIZE)
+            {
+              LOG_ERROR ("Invalid section->pages[%d].*size", i);
+              section->num_pages = i; // skip this last section
+              return sections;
+            }
           assert (section->pages[i]->size < DBG_MAX_SIZE);
           assert (section->pages[i]->uncomp_size < DBG_MAX_SIZE);
           assert (section->pages[i]->comp_size < DBG_MAX_SIZE);
@@ -994,6 +1033,13 @@ read_pages_map (Bit_Chain *dat, int64_t size_comp, int64_t size_uncomp,
           pages_destroy (pages);
           return NULL;
         }
+      if (ptr + 16 > ptr_end)
+        {
+          LOG_ERROR ("Page out of bounds")
+          free (data);
+          pages_destroy (pages);
+          return NULL;
+        }
 
       page->size = bfr_read_uint64 (ptr);
       page->id = bfr_read_int64 (ptr);
@@ -1106,12 +1152,13 @@ read_file_header (Bit_Chain *restrict dat,
   int32_t compr_len, len2;
   int i;
   int error = 0, errcount = 0;
+  const int pedata_size = 3 * 239; // size of pedata
 
   dat->byte = 0x80;
   LOG_TRACE ("\n=== File header ===\n")
   memset (file_header, 0, sizeof (r2007_file_header));
   bit_read_fixed (dat, data, 0x3d8);
-  pedata = decode_rs (data, 3, 239);
+  pedata = decode_rs (data, 3, 239, 0x3d8);
   if (!pedata)
     return DWG_ERR_OUTOFMEM;
 
@@ -1129,7 +1176,7 @@ read_file_header (Bit_Chain *restrict dat,
 
   if (compr_len > 0)
     error = decompress_r2007 ((BITCODE_RC *)file_header, 0x110, &pedata[32],
-                              compr_len);
+                              MIN (compr_len, pedata_size - 32));
   else
     memcpy (file_header, &pedata[32], sizeof (r2007_file_header));
 
@@ -1239,12 +1286,6 @@ obj_string_stream (Bit_Chain *restrict dat, Dwg_Object *restrict obj,
   str->size = (obj->bitsize / 8) + ((obj->bitsize % 8) ? 1 : 0);
   bit_advance_position (str, start - 8);
 
-  if (obj->supertype == DWG_SUPERTYPE_UNKNOWN)
-    {
-      str->size = 0;
-      // bit_reset_chain (str);
-      return 0;
-    }
   if (str->byte >= old_size - old_byte)
     {
       LOG_WARN ("obj_string_stream overflow");
@@ -1425,8 +1466,8 @@ read_2007_section_classes (Bit_Chain *restrict dat, Dwg_Data *restrict dwg,
           dwg->dwg_class[i].appname = (char *)bit_read_TU (&str);
           dwg->dwg_class[i].cppname = (char *)bit_read_TU (&str);
           dwg->dwg_class[i].dxfname_u = bit_read_TU (&str);
-          dwg->dwg_class[i].wasazombie = bit_read_B (&sec_dat);
-          dwg->dwg_class[i].item_class_id = bit_read_BS (&sec_dat);
+          dwg->dwg_class[i].is_zombie = bit_read_B (&sec_dat); // DXF 280
+          dwg->dwg_class[i].item_class_id = bit_read_BS (&sec_dat); // DXF 281, is_entity is 0x1f3
 
           dwg->dwg_class[i].num_instances = bit_read_BL (&sec_dat); // DXF 91
           dwg->dwg_class[i].dwg_version = bit_read_BS (&sec_dat);
@@ -1541,7 +1582,7 @@ read_2007_section_handles (Bit_Chain *dat, Bit_Chain *hdl,
 
   error = read_data_section (&obj_dat, dat, sections_map, pages_map,
                              SECTION_OBJECTS);
-  if (error >= DWG_ERR_CRITICAL)
+  if (error >= DWG_ERR_CRITICAL || !obj_dat.chain)
     {
       LOG_ERROR ("Failed to read objects section");
       if (obj_dat.chain)
@@ -1552,7 +1593,7 @@ read_2007_section_handles (Bit_Chain *dat, Bit_Chain *hdl,
   LOG_TRACE ("\nHandles\n-------------------\n")
   error = read_data_section (&hdl_dat, dat, sections_map, pages_map,
                              SECTION_HANDLES);
-  if (error >= DWG_ERR_CRITICAL)
+  if (error >= DWG_ERR_CRITICAL|| !hdl_dat.chain)
     {
       LOG_ERROR ("Failed to read handles section");
       if (obj_dat.chain)
@@ -1583,7 +1624,7 @@ read_2007_section_handles (Bit_Chain *dat, Bit_Chain *hdl,
         }
 
       last_offset = 0;
-      while (hdl_dat.byte - startpos < section_size)
+      while ((long)(hdl_dat.byte - startpos) < (long)section_size)
         {
           int added;
           BITCODE_UMC handleoff;
@@ -1656,7 +1697,7 @@ read_2007_section_summary (Bit_Chain *restrict dat, Dwg_Data *restrict dwg,
   old_dat = *dat;
   error = read_data_section (&obj_dat, dat, sections_map, pages_map,
                              SECTION_SUMMARYINFO);
-  if (error >= DWG_ERR_CRITICAL)
+  if (error >= DWG_ERR_CRITICAL || !obj_dat.chain)
     {
       LOG_ERROR ("Failed to read SummaryInfo section");
       if (obj_dat.chain)
@@ -1704,6 +1745,13 @@ read_2007_section_preview (Bit_Chain *restrict dat, Dwg_Data *restrict dwg,
     LOG_WARN ("thumbnail_address mismatch: " FORMAT_RL " != %lu",
               dwg->header.thumbnail_address, dat->byte);
   LOG_TRACE ("\nPreview\n-------------------\n")
+  if (!sec_dat.chain || sec_dat.size < 16)
+    {
+      LOG_WARN ("Empty thumbnail");
+      if (sec_dat.chain)
+        free (sec_dat.chain);
+      return error;
+    }
 
   sentinel = dwg_sentinel (DWG_SENTINEL_THUMBNAIL_BEGIN);
   if (memcmp (sentinel, sec_dat.chain, 16))
@@ -1729,10 +1777,10 @@ read_2007_section_preview (Bit_Chain *restrict dat, Dwg_Data *restrict dwg,
 
 /* exported */
 void
-read_r2007_init (Dwg_Data *dwg)
+read_r2007_init (Dwg_Data *restrict dwg)
 {
   if (dwg->opts)
-    loglevel = dwg->opts & 0xf;
+    loglevel = dwg->opts & DWG_OPTS_LOGLEVEL;
 }
 
 int
@@ -1794,8 +1842,8 @@ read_r2007_meta_data (Bit_Chain *dat, Bit_Chain *hdl_dat,
   if (!sections_map)
     goto error;
 
-  error
-      = read_2007_section_header (dat, hdl_dat, dwg, sections_map, pages_map);
+  error = read_2007_section_header (dat, hdl_dat, dwg, sections_map,
+                                    pages_map);
   if (dwg->header.summaryinfo_address)
     error |= read_2007_section_summary (dat, dwg, sections_map, pages_map);
   error |= read_2007_section_classes (dat, dwg, sections_map, pages_map);
