@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2009-2010,2018-2019 Free Software Foundation, Inc.         */
+/*  Copyright (C) 2009-2010,2018-2020 Free Software Foundation, Inc.         */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -36,8 +36,13 @@
 #include "decode.h"
 #include "dwg.h"
 #include "hash.h"
-#include "encode.h"
-#include "in_dxf.h"
+#include "dynapi.h"
+#ifdef USE_WRITE
+#  include "encode.h"
+#endif
+#ifndef DISABLE_DXF
+#  include "in_dxf.h"
+#endif
 #include "free.h"
 
 /* The logging level per .o */
@@ -152,7 +157,7 @@ dwg_read_file (const char *restrict filename, Dwg_Data *restrict dwg)
   Bit_Chain bit_chain = { 0 };
   int error;
 
-  loglevel = dwg->opts;
+  loglevel = dwg->opts & DWG_OPTS_LOGLEVEL;
   memset (dwg, 0, sizeof (Dwg_Data));
   dwg->opts = loglevel;
 
@@ -222,9 +227,7 @@ dwg_read_file (const char *restrict filename, Dwg_Data *restrict dwg)
   return error;
 }
 
-/* if write support is enabled */
-#if defined(USE_WRITE) && !defined(DISABLE_DXF)
-
+#if !defined(DISABLE_DXF) && defined(USE_WRITE)
 /** dxf_read_file
  * returns 0 on success.
  *
@@ -241,7 +244,7 @@ dxf_read_file (const char *restrict filename, Dwg_Data *restrict dwg)
   size_t size;
   Bit_Chain dat = { 0 };
 
-  loglevel = dwg->opts;
+  loglevel = dwg->opts & DWG_OPTS_LOGLEVEL;
 
   if (!filename || stat (filename, &attrib))
     {
@@ -267,10 +270,10 @@ dxf_read_file (const char *restrict filename, Dwg_Data *restrict dwg)
   /* Load whole file into memory
    */
   memset (dwg, 0, sizeof (Dwg_Data));
-  dwg->opts = loglevel | 0x2f;
+  dwg->opts = loglevel | DWG_OPTS_INDXF;
   memset (&dat, 0, sizeof (Bit_Chain));
   dat.size = attrib.st_size;
-  dat.chain = (unsigned char *)calloc (1, dat.size);
+  dat.chain = (unsigned char *)calloc (1, dat.size + 2);
   if (!dat.chain)
     {
       LOG_ERROR ("Not enough memory.\n")
@@ -281,6 +284,7 @@ dxf_read_file (const char *restrict filename, Dwg_Data *restrict dwg)
   dat.bit = 0;
   dat.from_version = dwg->header.from_version;
   dat.version = dwg->header.version;
+  dat.opts = dwg->opts;
 
   size = fread (dat.chain, sizeof (char), dat.size, fp);
   if (size != dat.size)
@@ -294,6 +298,9 @@ dxf_read_file (const char *restrict filename, Dwg_Data *restrict dwg)
       return DWG_ERR_IOERROR;
     }
   fclose (fp);
+  // properly end the buffer for strtol()/... readers
+  dat.chain[size] = '\n';
+  dat.chain[size + 1] = '\0';
 
   /* Fail on DWG */
   if (!memcmp (dat.chain, "AC10", 4))
@@ -311,7 +318,7 @@ dxf_read_file (const char *restrict filename, Dwg_Data *restrict dwg)
   else
     error = dwg_read_dxf (&dat, dwg);
 
-  dwg->opts |= 0x2f;
+  dwg->opts |= (DWG_OPTS_INDXF | loglevel);
   if (error >= DWG_ERR_CRITICAL)
     {
       LOG_ERROR ("Failed to decode DXF file: %s\n", filename)
@@ -328,7 +335,9 @@ dxf_read_file (const char *restrict filename, Dwg_Data *restrict dwg)
 
   return 0;
 }
+#endif /* DISABLE_DXF */
 
+#ifdef USE_WRITE
 EXPORT int
 dwg_write_file (const char *restrict filename, const Dwg_Data *restrict dwg)
 {
@@ -337,7 +346,7 @@ dwg_write_file (const char *restrict filename, const Dwg_Data *restrict dwg)
   Bit_Chain dat = { 0 };
   int error;
 
-  loglevel = dwg->opts;
+  loglevel = dwg->opts & DWG_OPTS_LOGLEVEL;
   assert (filename);
   assert (dwg);
   dat.opts = dwg->opts;
@@ -360,7 +369,13 @@ dwg_write_file (const char *restrict filename, const Dwg_Data *restrict dwg)
     }
 
   // try opening the output file in write mode
-  if (!stat (filename, &attrib))
+  if (!stat (filename, &attrib)
+#ifdef _WIN32
+      && strNE (filename, "NUL")
+#else
+      && strNE (filename, "/dev/null")
+#endif
+      )
     {
       LOG_ERROR ("The file already exists. We won't overwrite it.")
       return error | DWG_ERR_IOERROR;
@@ -407,7 +422,7 @@ dwg_bmp (const Dwg_Data *restrict dwg, BITCODE_RL *restrict size)
   BITCODE_RL header_size, address, osize;
   Bit_Chain *dat;
 
-  loglevel = dwg->opts;
+  loglevel = dwg->opts & DWG_OPTS_LOGLEVEL;
   *size = 0;
   assert (dwg);
   dat = (Bit_Chain *)&dwg->thumbnail;
@@ -927,6 +942,7 @@ get_next_owned_entity (const Dwg_Object *restrict hdr,
           obj = dwg_next_object (obj);
           // this may happen with r2000 attribs
           if (obj && obj->supertype == DWG_SUPERTYPE_ENTITY
+              && obj->tio.entity != NULL
               && obj->tio.entity->ownerhandle != NULL
               && obj->tio.entity->ownerhandle->absolute_ref
                      != hdr->handle.value)
@@ -1187,9 +1203,9 @@ get_last_owned_block (const Dwg_Object *restrict hdr)
 }
 
 EXPORT int
-dwg_class_is_entity (const Dwg_Class *klass)
+dwg_class_is_entity (const Dwg_Class *restrict klass)
 {
-  return klass && klass->item_class_id == 0x1f2;
+  return (klass != NULL && (int)klass->item_class_id == 0x1f2) ? 1 : 0;
 }
 
 EXPORT int
@@ -1502,7 +1518,7 @@ dwg_find_table_control (Dwg_Data *restrict dwg, const char *restrict table)
   return NULL;
 }
 
-// Searching for a named dictionary entry.
+// Searching for a named dictionary entry, name is utf8.
 // Returning a hardpointer ref (5) to it, as stored in header_vars.
 // Usually another dictionary.
 EXPORT BITCODE_H
@@ -1518,12 +1534,15 @@ dwg_find_dictionary (Dwg_Data *restrict dwg, const char *restrict name)
           Dwg_Object_DICTIONARY *_obj = obj->tio.object->tio.DICTIONARY;
           for (BITCODE_BL j = 0; j < _obj->numitems; j++)
             {
-              char *u8 = _obj->texts[j];
+              char *u8;
+              if (!_obj->texts || !_obj->itemhandles)
+                continue;
+              u8 = _obj->texts[j];
               if (!u8)
                 continue;
               if (dwg->header.version >= R_2007)
                 u8 = bit_convert_TU ((BITCODE_TU)u8);
-              if (strEQ (u8, name))
+              if (u8 && strEQ (u8, name))
                 {
                   Dwg_Object_Ref *ref = _obj->itemhandles[j];
                   if (!ref)
@@ -1550,7 +1569,7 @@ dwg_find_dicthandle (Dwg_Data *restrict dwg, BITCODE_H dict, const char *restric
   Dwg_Object_DICTIONARY *_obj;
   Dwg_Object *obj = dwg_resolve_handle (dwg, dict->absolute_ref);
 
-  if (!obj)
+  if (!obj || !obj->tio.object)
     {
       LOG_TRACE ("dwg_find_dicthandle: Could not resolve dict " FORMAT_REF "\n",
                  ARGS_REF(dict));
@@ -1571,20 +1590,26 @@ dwg_find_dicthandle (Dwg_Data *restrict dwg, BITCODE_H dict, const char *restric
       BITCODE_H *hdlv = _obj->itemhandles;
       Dwg_Object *hobj;
       Dwg_Object_APPID *_o; // just some random type
+      int isnew = 0;
+      bool ok;
 
-      if (!hdlv[i])
+      if (!hdlv | !hdlv[i])
         continue;
       hobj = dwg_resolve_handle (dwg, hdlv[i]->absolute_ref);
-      if (!hobj)
+      if (!hobj || !hobj->tio.object || !hobj->tio.object->tio.APPID || !hobj->name)
         continue;
       _o = hobj->tio.object->tio.APPID;
-      dwg_dynapi_entity_utf8text (_o, hobj->name, "name", &hdlname, NULL);
+      ok = dwg_dynapi_entity_utf8text (_o, hobj->name, "name", &hdlname, &isnew, NULL);
       LOG_HANDLE (" %s.%s[%d] => %s.name: %s\n", obj->name, "entries", i,
                   hobj->name, hdlname ? hdlname : "NULL");
-      if (hdlname && (strEQ (name, hdlname) || !strcasecmp (name, hdlname)))
+      if (ok && hdlname && (strEQ (name, hdlname) || !strcasecmp (name, hdlname)))
         {
+          if (isnew)
+            free (hdlname);
           return hdlv[i];
         }
+      if (ok && isnew && hdlname)
+        free (hdlname);
     }
   return NULL;
 }
@@ -1595,9 +1620,13 @@ dwg_find_tablehandle_silent (Dwg_Data *restrict dwg, const char *restrict name,
 {
   BITCODE_H ref;
   int oldopts = dwg->opts;
-  dwg->opts = loglevel = dwg->opts & ~0xf;
+  dwg->opts &= ~(DWG_OPTS_LOGLEVEL);
+  loglevel = 0;
+
   ref = dwg_find_tablehandle (dwg, name, table);
-  dwg->opts = loglevel = oldopts;
+
+  dwg->opts = oldopts;
+  loglevel = oldopts & DWG_OPTS_LOGLEVEL;
   return ref;
 }
 
@@ -1610,11 +1639,13 @@ dwg_find_tablehandle (Dwg_Data *restrict dwg, const char *restrict name,
                       const char *restrict table)
 {
   BITCODE_BL i, num_entries = 0;
-  BITCODE_H ctrl = NULL, *hdlv;
+  BITCODE_H ctrl = NULL, *hdlv = NULL;
   Dwg_Object *obj;
   Dwg_Object_APPID_CONTROL *_obj; // just some random generic type
   Dwg_Header_Variables *vars = &dwg->header_vars;
 
+  if (!dwg || !name || !table)
+    return NULL;
   // look for the _CONTROL table, and search for name in all entries
   if (strEQc (table, "BLOCK"))
     {
@@ -1789,25 +1820,33 @@ dwg_find_tablehandle (Dwg_Data *restrict dwg, const char *restrict name,
   if (!num_entries)
     return 0;
   dwg_dynapi_entity_value (_obj, obj->name, "entries", &hdlv, NULL);
+  if (!hdlv)
+    return 0;
   for (i = 0; i < num_entries; i++)
     {
       char *hdlname;
       Dwg_Object *hobj;
       Dwg_Object_APPID *_o;
+      int isnew = 0;
+      bool ok;
 
       if (!hdlv[i])
         continue;
       hobj = dwg_resolve_handle (dwg, hdlv[i]->absolute_ref);
-      if (!hobj)
+      if (!hobj || !hobj->tio.object || !hobj->tio.object->tio.APPID)
         continue;
       _o = hobj->tio.object->tio.APPID;
-      dwg_dynapi_entity_utf8text (_o, hobj->name, "name", &hdlname, NULL);
+      ok = dwg_dynapi_entity_utf8text (_o, hobj->name, "name", &hdlname, &isnew, NULL);
       LOG_HANDLE (" %s.%s[%d] => %s.name: %s\n", obj->name, "entries", i,
                   hobj->name, hdlname ? hdlname : "NULL");
-      if (hdlname && (strEQ (name, hdlname) || !strcasecmp (name, hdlname)))
+      if (ok && hdlname && (strEQ (name, hdlname) || !strcasecmp (name, hdlname)))
         {
+          if (isnew)
+            free (hdlname);
           return hdlv[i];
         }
+      if (ok && isnew && hdlname)
+        free (hdlname);
     }
 
   return 0;
@@ -1836,11 +1875,15 @@ is_extnames_xrecord (Dwg_Data *restrict dwg, Dwg_Object *restrict xrec,
   const int16_t w[8] = { 'E', 'X', 'T', 'N', 'A', 'M', 'E', 'S' };
   const char *extnames = dwg->header.from_version < R_2007 ? "EXTNAMES" : (const char*)w;
 
-  return (xrec->fixedtype == DWG_TYPE_XRECORD
+  return (xrec
+          && xdic
+          && dwg
+          && xrec->fixedtype == DWG_TYPE_XRECORD
           && xrec->tio.object->ownerhandle
           && xrec->tio.object->ownerhandle->absolute_ref
-          == xdic->handle.value
+               == xdic->handle.value
           && xrec->tio.object->tio.XRECORD->num_xdata >= 2
+          && xrec->tio.object->tio.XRECORD->xdata
           && xdata_string_match (dwg, xrec->tio.object->tio.XRECORD->xdata,
                                  102, (char *)extnames));
 }
@@ -1883,7 +1926,7 @@ dwg_find_table_extname (Dwg_Data *restrict dwg, Dwg_Object *restrict obj)
   if (i == _xdic->numitems) // not found
     return NULL;
   xrec = dwg_ref_object (dwg, _xdic->itemhandles[i]);
-  if (!is_extnames_xrecord (dwg, xrec, xdic))
+  if (!xrec || !is_extnames_xrecord (dwg, xrec, xdic))
     return NULL;
 
   _xrec = xrec->tio.object->tio.XRECORD;
