@@ -24,6 +24,6 @@
 EXPORT int dwg_write_json (Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
 EXPORT int dwg_write_geojson (Bit_Chain *restrict dat, Dwg_Data *restrict dwg);
 
-char *json_cquote (char *restrict dest, const char *restrict src);
+char *json_cquote (char *restrict dest, const char *restrict src, const int len);
 
 #endif
