@@ -1,7 +1,7 @@
 /*****************************************************************************/
 /*  LibreDWG - free implementation of the DWG file format                    */
 /*                                                                           */
-/*  Copyright (C) 2018-2019 Free Software Foundation, Inc.                   */
+/*  Copyright (C) 2018-2020 Free Software Foundation, Inc.                   */
 /*                                                                           */
 /*  This library is free software, licensed under the terms of the GNU       */
 /*  General Public License as published by the Free Software Foundation,     */
@@ -156,6 +156,10 @@
 #ifndef FIELD_BINARY
 #  define FIELD_BINARY(name, len, dxf) FIELD_TF (name, len, dxf)
 #endif
+#ifndef LOG_TRACE_TF
+#  define LOG_TRACE_TF(var, len)
+#  define LOG_INSANE_TF(var, len)
+#endif
 #ifndef SUBCLASS
 #  define SUBCLASS(text)
 #endif
@@ -171,10 +175,6 @@
                      ") [" #type " %d]\n",                                    \
                _obj->name.x, _obj->name.y, _obj->name.z, dxf)
 #endif
-#ifndef LOG_TRACE_TF
-#  define LOG_TRACE_TF(var, len)
-#  define LOG_INSANE_TF(var, len)
-#endif
 #ifndef FIELD_ENC
 #  define FIELD_ENC(a, b, c) FIELD_CMC (a, b, c)
 #endif
@@ -198,7 +198,7 @@
 #  define IF_ENCODE_FROM_EARLIER                                              \
     if (dat->from_version && dat->from_version < cur_ver)
 #  define IF_ENCODE_FROM_EARLIER_OR_DXF                                       \
-    if ((dat->from_version && dat->from_version < cur_ver) || dwg->opts & 0x20)
+    if ((dat->from_version && dat->from_version < cur_ver) || dwg->opts & DWG_OPTS_INDXF)
 #  define IF_ENCODE_FROM_PRE_R13                                              \
     if (dat->from_version && dat->from_version < R_13)
 #  define IF_ENCODE_SINCE_R13                                                 \
@@ -375,13 +375,13 @@
                    (long)_obj->times);                                        \
         return DWG_ERR_VALUEOUTOFBOUNDS;                                      \
       }                                                                       \
-    if (_obj->name)                                                           \
+    if (_obj->times && _obj->name)                                            \
       for (rcount##idx = 0; rcount##idx < (BITCODE_BL)_obj->times;            \
            rcount##idx++)
 // unchecked with var. times
 #  ifndef _REPEAT_C
 #    define _REPEAT_C(times, name, type, idx)                                 \
-      if (_obj->name)                                                         \
+      if (_obj->times && _obj->name)                                          \
         for (rcount##idx = 0; rcount##idx < (BITCODE_BL)_obj->times;          \
              rcount##idx++)
 #  endif
